/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Calendar;

/**
 * File handling with error support. This may be used for non critical
 * applications.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class FileSupport {

	/**
	 * Provide the file list on all sub-directories of a specific directory.
	 * 
	 * @param file
	 *            The file or directory.
	 * @param files
	 *            The array to store the files.
	 * @param sufixes
	 *            The suffixes to be include. If it is null than include all
	 *            file types.
	 * @return The array with all the files on the sub-directories.
	 */
	private static ArrayList retrieveDirectories(File file[], ArrayList files,
			String suffixes[]) {

		for (int i = 0; i < file.length; i++) {

			if (file[i].isDirectory()) {
				retrieveDirectories(file[i].listFiles(), files, suffixes);
			} else {

				// Verify the suffix.
				String fileName = file[i].getAbsoluteFile().toString();

				// If not suffix is provided than add all files.
				if (suffixes == null || "".equals(suffixes)) {

					files.add(fileName);

				} else {

					for (int j = 0; j < suffixes.length; j++) {

						if (fileName.endsWith(suffixes[j])) {

							files.add(fileName);
						}
					}
				}
			}
		}

		return files;
	}

	/**
	 * Provide the file list on all sub-directories of a specific directory.
	 * 
	 * @param directory
	 *            The directory to be searched.
	 * @param sufixes
	 *            The suffixes to be include. If it is null than include all
	 *            file types.
	 * @return The array with all the files on the sub-directories.
	 */
	public static String[] getFileOnSubfolder(String directory,
			String suffixes[]) {

		ArrayList files = new ArrayList();
		files = retrieveDirectories(new File[] { new File(directory) }, files,
				suffixes);

		String[] returnFiles = new String[files.size()];
		return (String[]) files.toArray(returnFiles);
	}

	/**
	 * Generate a sequence of directories
	 * 
	 * @param directory
	 *            The target directories.
	 * @return The sequence of directories.
	 */
	private static String[] getSequenceOfDirectories(String directory) {

		String targetDirectory = new String();

		ArrayList directories = new ArrayList();

		for (int i = 0; i < directory.length(); i++) {

			char thisCharacter = directory.charAt(i);

			if ('/' == thisCharacter || '\\' == thisCharacter) {
				directories.add(new String(targetDirectory));
			}

			targetDirectory += thisCharacter;

		}
		directories.add(new String(targetDirectory));

		String[] returnDirectoreis = new String[directories.size()];
		returnDirectoreis = (String[]) directories.toArray(returnDirectoreis);

		return returnDirectoreis;
	}

	/**
	 * Provide only the path of an absolute file name.
	 * 
	 * @param absoluteFileName
	 *            The absolute file name.
	 * @return The path.
	 */
	public static String getOnlyPath(String absoluteFileName) {

		int lastDirectoryPosition = 0;

		for (int i = 0; i < absoluteFileName.length(); i++) {

			char thisCharacter = absoluteFileName.charAt(i);

			if ('/' == thisCharacter || '\\' == thisCharacter) {
				lastDirectoryPosition = i;
			}
		}

		return absoluteFileName.substring(0, lastDirectoryPosition);
	}

	/**
	 * Indicate that the listOfFiles being
	 * 
	 * @param directoryName
	 * @return true if the creation was successful and false otherwise.
	 */
	public static boolean createDirectory(String directoryName) {

		String directories[] = getSequenceOfDirectories(directoryName);

		for (int i = 0; i < directories.length; i++) {

			String directory = directories[i];

			File thisDirectory = new File(directory);

			if (!thisDirectory.exists()) {

				if (!thisDirectory.mkdir()) {
					return false;
				}
			}

		}

		return true;
	}

	/**
	 * Read a text file.
	 * 
	 * @param fileName
	 *            The file name.
	 * @return The content or null if there is a problem.
	 */
	public static StringBuffer readTextFile(String fileName) {

		File file = new File(fileName);

		FileReader fileReader;
		try {
			fileReader = new FileReader(file);
		} catch (FileNotFoundException e) {
			ExceptionSupport.handleException(e);
			return null;
		}

		BufferedReader reader = new BufferedReader(fileReader);

		StringBuffer content = new StringBuffer();

		String line;
		do {

			try {
				line = reader.readLine();
			} catch (IOException e) {
				ExceptionSupport.handleException(e);
				return null;
			}

			if (line != null) {

				content.append(line);
				content.append(CommonConstants.NEW_LINE);
			}

		} while (line != null);

		try {
			reader.close();
		} catch (IOException e) {
			ExceptionSupport.handleException(e);
		}

		return content;
	}

	/**
	 * Inform if a file exist.
	 * 
	 * @param fileName
	 *            The file name.
	 * @return True if the file exists, false otherwise.
	 */
	public static boolean fileExist(String fileName) {

		File file = new File(fileName);

		return file.exists();
	}

	/**
	 * Write text to file.
	 * 
	 * @param fileName
	 *            The file name.
	 * @param content
	 *            The content of the file.
	 * @return null if no problem or a message is error.
	 */
	public static String writeTextFile(String fileName, String content) {

		File file = new File(fileName);

		if (file.exists()) {
			if (!file.delete()) {
				ExceptionSupport.handleException(InternalResource.get(
						InternalResource.EXCEPTION_COULD_NO_DELETE, fileName));
				return InternalResource.get(
						InternalResource.EXCEPTION_COULD_NO_DELETE, fileName);
			}
		}

		FileWriter fileReader;
		try {
			fileReader = new FileWriter(file);
		} catch (IOException exception) {
			ExceptionSupport.handleException(exception);
			return exception.getLocalizedMessage();
		}

		BufferedWriter writer = new BufferedWriter(fileReader);

		try {
			writer.write(content);
			writer.close();
		} catch (IOException exception) {
			ExceptionSupport.handleException(exception);
			return exception.getLocalizedMessage();
		}

		return null;
	}

	/**
	 * Read a binary file.
	 * 
	 * @param fileName
	 *            The file name.
	 * @return The content or null if there is a problem.
	 */
	public static byte[] readBinaryFile(String fileName) {

		File file = new File(fileName);

		FileInputStream stream = null;

		try {
			stream = new FileInputStream(file);
		} catch (FileNotFoundException e) {
			ExceptionSupport.handleException(e);
			return null;
		}

		DataInputStream data = new DataInputStream(stream);

		long size = file.length();

		if (size > 550000000L) {
			ExceptionSupport.handleException(InternalResource.get(
					InternalResource.FILE_TOO_BIG, file.getName()));
			return null;

		}

		byte content[] = new byte[(int) size];

		try {
			data.readFully(content);
		} catch (IOException e) {
			ExceptionSupport.handleException(e);
			return null;
		}

		return content;
	}

	/**
	 * Write a binary file.
	 * 
	 * @param fileName
	 *            The file name.
	 * @param content
	 *            The file content.
	 */
	public static void writeBinaryFile(String fileName, byte content[]) {

		File file = new File(fileName);

		FileOutputStream stream = null;

		try {
			stream = new FileOutputStream(file);
		} catch (FileNotFoundException e) {
			ExceptionSupport.handleException(e);
			return;
		}

		DataOutputStream data = new DataOutputStream(stream);

		try {
			data.write(content);
		} catch (IOException e) {
			ExceptionSupport.handleException(e);
			return;
		}
	}

	/**
	 * Provide the file name along with the directory.
	 * 
	 * @param directoryName
	 *            the directory name.
	 * @param fileName
	 *            the file name.
	 * @return The file name along with the directory.
	 */
	public static String getFileName(String directoryName, String fileName) {

		File directory = new File(directoryName);

		return directory.getAbsolutePath() + "/" + fileName;
	}

	/**
	 * A variation to write a file to a specific directory.
	 * 
	 * @param directoryName
	 *            The directory name.
	 * @param fileName
	 *            The file name.
	 * @param content
	 *            The content of the file.
	 * @return null if no problem or a message is error.
	 */
	public static String writeTextFile(String directoryName, String fileName,
			String content) {

		return writeTextFile(getFileName(directoryName, fileName), content);
	}

	/**
	 * Read a text listOfFiles and return an array of strings each one
	 * representing a line.
	 * 
	 * @param fileName
	 *            The file name.
	 * @return The lines of the file.
	 */
	public static String[] readAllLines(String fileName) {

		File file = new File(fileName);

		FileReader fileReader;
		try {
			fileReader = new FileReader(file);
		} catch (FileNotFoundException e) {
			ExceptionSupport.handleException(e);
			return null;
		}

		BufferedReader reader = new BufferedReader(fileReader);

		ArrayList lines = new ArrayList();

		String line;
		do {

			try {
				line = reader.readLine();
			} catch (IOException e) {
				ExceptionSupport.handleException(e);
				return null;
			}

			if (line != null) {

				lines.add(line);
			}

		} while (line != null);

		String resultingLines[] = new String[lines.size()];
		for (int i = 0; i < lines.size(); i++) {
			resultingLines[i] = (String) lines.get(i);
		}

		try {
			reader.close();
		} catch (IOException e) {
			ExceptionSupport.handleException(e);
		}

		return resultingLines;
	}

	/**
	 * Retrieve the list of files from a specific directory
	 * 
	 * @param directory
	 *            The directory name
	 * @param sufixes
	 *            The suffixes to be include. If it is null than include all
	 *            file types.
	 * @return The array of file names or null if error.
	 */
	public static String[] getFilesFromDirectory(String directory,
			String suffixes[]) {

		File searchDirectory = new File(directory);

		if (!searchDirectory.isDirectory()) {
			return null;
		}

		File fileList[] = searchDirectory.listFiles();

		ArrayList searchFiles = new ArrayList();
		for (int j = 0; j < fileList.length; j++) {

			if (!fileList[j].isDirectory()) {

				// Verify the suffix.
				String fileName = fileList[j].getAbsoluteFile().toString();

				// If not suffix is provided than add all files.
				if (suffixes == null || "".equals(suffixes)) {

					searchFiles.add(fileName);

				} else {

					// Only include the ones with the suffix.
					for (int k = 0; k < suffixes.length; k++) {

						if (fileName.endsWith(suffixes[k])) {

							searchFiles.add(fileName);
							break;
						}
					}
				}
			}
		}

		String fileNames[] = new String[searchFiles.size()];
		fileNames = (String[]) searchFiles.toArray(fileNames);

		return fileNames;
	}

	/**
	 * Generate a string with the current date and time.
	 * 
	 * @return The current date to be used in a file name.
	 */
	public static String getCurrentDateForFileName() {

		Calendar calendar = Calendar.getInstance();

		String month = (calendar.get(Calendar.MONTH) + 1) < 10 ? "0"
				+ (calendar.get(Calendar.MONTH) + 1) : ""
				+ (calendar.get(Calendar.MONTH) + 1);
		String day = (calendar.get(Calendar.DAY_OF_MONTH) < 10) ? "0"
				+ calendar.get(Calendar.DAY_OF_MONTH) : ""
				+ calendar.get(Calendar.DAY_OF_MONTH);
		String hour = calendar.get(Calendar.HOUR_OF_DAY) < 10 ? "0"
				+ calendar.get(Calendar.HOUR_OF_DAY) : ""
				+ calendar.get(Calendar.HOUR_OF_DAY);
		String minute = calendar.get(Calendar.MINUTE) < 10 ? "0"
				+ calendar.get(Calendar.MINUTE) : ""
				+ calendar.get(Calendar.MINUTE);
		String second = calendar.get(Calendar.SECOND) < 10 ? "0"
				+ calendar.get(Calendar.SECOND) : ""
				+ calendar.get(Calendar.SECOND);

		return "" + calendar.get(Calendar.YEAR) + month + day + "_" + hour
				+ minute + second;

	}

	/**
	 * Combine a directory name with a file name also on a directory.
	 * 
	 * @param directory
	 *            The target directory.
	 * @param file
	 *            The file within a directory.
	 * @return The combination of the directory and the file.
	 */
	public static String getCombinedDirectory(String directory, String file) {

		// If there is no directory than return the file itself.
		if (directory == null || "".equals(directory)) {
			return file;
		}

		// Remove the slash at the end of the directory.
		if (directory.charAt(directory.length() - 1) == '\\'
				|| directory.charAt(directory.length() - 1) == '/') {
			directory = directory.substring(0, directory.length() - 1);
		}

		// The file is a required field.
		if (file == null || "".equals(file)) {
			throw new InvalidParameterException(InternalResource
					.get(InternalResource.EXCEPTION_MISSING_FILE_NAME));
		}

		// Remove the disk information
		if (file.charAt(1) == ':') {
			file = file.substring(2);
		}

		return directory + file;
	}

	/**
	 * Remove the files from a specific directory.
	 * 
	 * @param directory
	 *            The target directory.
	 * @return The list of files removed successfully.
	 */
	public static String[] deleteFilesFromDirectory(String directory) {

		// Clear output directory.
		String fileNames[] = FileSupport.getFilesFromDirectory(directory, null);
		ArrayList filesRemoved = new ArrayList();

		for (int i = 0; i < fileNames.length; i++) {

			String fileName = fileNames[i];

			File file = new File(fileName);
			// Only remove files.
			if (file.isFile()) {

				if (!file.delete()) {
					ExceptionSupport.handleException(InternalResource.get(
							InternalResource.EXCEPTION_COULD_NO_DELETE,
							fileName));
				} else {
					filesRemoved.add(file.getName());
				}
			}
		}

		String removedFileNames[] = new String[filesRemoved.size()];

		return (String[]) filesRemoved.toArray(removedFileNames);
	}

	/**
	 * Remove all the files and directory from a specific location.
	 * 
	 * @param element
	 *            The initial directory.
	 * @return The files and directories that were removed.
	 */
	public static ArrayList deleteDirectoryTree(String element) {

		File fileOrDirectory = new File(element);
		ArrayList elementsRemoved = new ArrayList();

		// Not a valid input.
		if (!fileOrDirectory.isDirectory() && !fileOrDirectory.isFile()) {
			
			return elementsRemoved;
		}

		// Remove all files from directory.
		if (fileOrDirectory.isDirectory()) {

			String filesRemoved[] = deleteFilesFromDirectory(fileOrDirectory
					.getAbsolutePath());
			ArrayList filesRemovedArray = ArraySupport
					.toArrayList(filesRemoved);
			elementsRemoved.addAll(filesRemovedArray);

		}

		File remainingDirectoreis[] = fileOrDirectory.listFiles();

		if (remainingDirectoreis.length == 0) {

			if (!fileOrDirectory.delete()) {

				System.out.println("Could not delete directory: "
						+ fileOrDirectory.getAbsolutePath());

			}

			return elementsRemoved;

		} else {

			for (int i = 0; i < remainingDirectoreis.length; i++) {

				elementsRemoved
						.addAll(deleteDirectoryTree(remainingDirectoreis[i]
								.getAbsolutePath()));
			}

			return elementsRemoved;
		}
	}
}
