/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */
package com.carlosbcruz.util;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.Action;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

/**
 * Helper class to implement popup.
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class JTreeEventHelperAdapter extends MouseAdapter implements
		TreeSelectionListener {

	// Listeners to the user objects events.
	private ArrayList listeners = new ArrayList();

	// If it is listening to tree events.
	private JTree tree = null;

	// Popup menus that can be opened
	private JPopupMenu popupMenu = null;

	/**
	 * General constructor to no specific object. 
	 */
	public JTreeEventHelperAdapter() {
		super();
	}

	/**
	 * Listening to a tree events.
	 * @param tree The tree.
	 */
	public JTreeEventHelperAdapter(JTree tree) {
		super();

		this.tree = tree;
	}

	/**
	 * Add an action to the popup menu.
	 * @param action The action.
	 */
	public void addPopupAction(Action action) {

		// Create the popup menus, transaction and checks
		if (popupMenu == null) {
			popupMenu = new JPopupMenu();
		}
		JMenuItem menuItem = new JMenuItem(action);
		popupMenu.add(menuItem);
	}

	/* (no-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {

		maybeShowPopup(e, true);
	}

	/* (no-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {

		maybeShowPopup(e, false);
	}

	/**
	 * Controls the popup menu.
	 * @param event the mouse event passed
	 */
	private void maybeShowPopup(MouseEvent event, boolean isMousePressed) {

		if (event.isPopupTrigger()) {

			// If popup menu defined.
			if (popupMenu != null) {

				popupMenu
						.show(event.getComponent(), event.getX(), event.getY());
			}

		} else {

			// Disconsider other events to avoid duplicated notifications.
			if (!isMousePressed) {

				return;
			}

			// Only consider the main buttom.
			if (event.getButton() != MouseEvent.BUTTON1) {

				return;
			}

			// If listening to tree events.
			if (tree != null) {

				int rowSelected = tree.getRowForLocation(event.getX(),
						event.getY());

				// Only open popup if above an tree node
				if (rowSelected != -1) {

					TreePath path = tree.getPathForRow(rowSelected);
					fireThreePathSelection(path);
				}
			}
		}
	}

	/**
	 * Fire the event of a tree path selection.
	 * @param path The three path selection.
	 */
	private void fireThreePathSelection(TreePath path) {
		DefaultMutableTreeNode lastNode = (DefaultMutableTreeNode) path
				.getLastPathComponent();
		Object userObject = lastNode.getUserObject();

		// Fire events on user object events listeners.
		for (int i = 0; i < listeners.size(); i++) {

			JTreeEventHelperListener listener = (JTreeEventHelperListener) listeners
					.get(i);
			listener.eventOnUserObject(userObject);
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
	 */
	public void valueChanged(TreeSelectionEvent event) {

		TreePath treePath = event.getNewLeadSelectionPath();
		fireThreePathSelection(treePath);
	}

	/**
	 * Add a listener to events on user objects.
	 * @param listener The listener.
	 */
	public void addEventHelperListener(JTreeEventHelperListener listener) {

		listeners.add(listener);
	}
}
