/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.util;

import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * Provides general string manipulation support
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class StringSupport {

	/**
	 * Find all ocurrences positions of the specified token into the text
	 * provided.
	 * 
	 * @param text
	 *            The text to searh into
	 * @param token
	 *            the token to look for
	 * @return the positions where the token was fond.
	 */
	public static int[] findOcurrences(StringBuffer text, String token) {

		ArrayList ocurrences = new ArrayList();

		int lastOcurrence = 0;
		int currentIndex = -1;
		do {

			currentIndex = text.indexOf(token, lastOcurrence);

			// There is one occurrence than continue the search
			if (currentIndex != -1) {
				ocurrences.add(new Integer(currentIndex));
				lastOcurrence = currentIndex + 1;
			}

		} while (currentIndex != -1);

		int resultList[] = new int[ocurrences.size()];
		for (int i = 0; i < ocurrences.size(); i++) {
			resultList[i] = ((Integer) ocurrences.get(i)).intValue();
		}

		return resultList;
	}

	/**
	 * Break the text into an array of lines based on the a delimiter.
	 * 
	 * @param text
	 *            The text.
	 * @param delimiter
	 *            The delimiter.
	 * @return The lines in an array format.
	 */
	public static String[] breakContentBasedOnDelimiter(String text,
			String delimiter) {

		StringTokenizer tokenizer = new StringTokenizer(text, delimiter);
		ArrayList lines = new ArrayList();
		while (tokenizer.hasMoreTokens()) {
			String line = tokenizer.nextToken();
			lines.add(line);
		}

		String[] resultLines = new String[lines.size()];
		resultLines = (String[]) lines.toArray(resultLines);

		return resultLines;
	}

	/**
	 * @param content
	 *            The string to have it's length adjusted.
	 * @param size
	 *            The minimum length.
	 * @return The content with spaces appended at the end if necessary.
	 */
	public static String setMinimumLengthRight(String content, int size) {

		if (content != null && content.length() < size) {
			String toAppend = new String();
			for (int i = 0; i < size - content.length(); i++) {
				toAppend += " ";
			}
			return content + toAppend;
		}

		return content;
	}

	/**
	 * Remove all characters that are not allowed.
	 * 
	 * @param content
	 *            The string to be filtered.
	 * @param allowedChars
	 *            The allowed characters.
	 * @return The filtered String.
	 */
	public static String filterString(String content, char allowedChars[]) {

		String filteredString = new String();
		for (int i = 0; i < content.length(); i++) {

			char currentChar = content.charAt(i);

			boolean found = false;
			for (int j = 0; j < allowedChars.length; j++) {
				if (currentChar == allowedChars[j]) {
					found = true;
					break;
				}
			}

			if (found) {
				filteredString += String.valueOf(currentChar);
			}
		}

		return filteredString;

	}

	/**
	 * Replace a string inside a provided content.
	 * 
	 * @param content
	 *            The content.
	 * @param searchFor
	 *            The string to search for.
	 * @param replaceBy
	 *            The string to replace.
	 * @return The string replaced or the content itself if the string is not
	 *         found.
	 */
	public static String replace(String content, String searchFor,
			String replaceBy) {

		int index = content.indexOf(searchFor);

		if (index == -1) {
			return content;
		}

		String returnString = content.substring(0, index);
		returnString += replaceBy;
		returnString += content.substring(index + searchFor.length());

		return returnString;
	}

	/**
	 * Replace specific set of chars on a string.
	 * 
	 * @param content
	 *            The string to be modified.
	 * @param searchFor
	 *            The character to be replaced.
	 * @param replaceBy
	 *            The character to replace.
	 * @return The string with the characters replaced.
	 */
	public static String replaceCharacters(String content, char searchFor,
			char replaceBy) {

		String finalContent = new String();
		for (int i = 0; i < content.length(); i++) {

			char thisChar = content.charAt(i);

			if (thisChar == searchFor) {
				finalContent += String.valueOf(replaceBy);
			} else {
				finalContent += String.valueOf(thisChar);
			}
		}

		return finalContent;
	}

	/**
	 * Inform the characteristics of a specific text.
	 * 
	 * @author Carlos Fernando Bella Cruz - <a
	 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
	 */
	public class TextCharacteristics {

		public static final int DOS = 1;
		public static final int UNIX = 2;

		int length;
		int words;
		int lines;
		int lineType;

		/**
		 * Default constructor.
		 * 
		 * @param length
		 *            The length of the text.
		 * @param words
		 *            The number of words.
		 * @param lines
		 *            The number of lines.
		 * @param lineType
		 *            The type of new lines.
		 */
		public TextCharacteristics(int length, int words, int lines,
				int lineType) {
			super();
			this.length = length;
			this.words = words;
			this.lines = lines;
			this.lineType = lineType;
		}

		/**
		 * @return the length
		 */
		public int getLength() {
			return length;
		}

		/**
		 * @return the words
		 */
		public int getWords() {
			return words;
		}

		/**
		 * @return the lines
		 */
		public int getLines() {
			return lines;
		}

		/**
		 * @return the lineType
		 */
		public int getLineType() {
			return lineType;
		}
	}

	/**
	 * Analyze the text and provide statistics.
	 * 
	 * @param text
	 *            The text to be analyzed.
	 * @return The text characteristics.
	 */
	public static StringSupport.TextCharacteristics analyzeText(String text) {

		int words = 0;
		int newLines = 0;
		int carriageReturns = 0;
		int newLinesWithCarriageReturns = 0;
		boolean lastCharacterWasNewLine = false;
		boolean lastCharacterWasCarriageReturn = false;
		boolean insideWord = false;
		for (int i = 0; i < text.length(); i++) {

			char currentChar = text.charAt(i);

			if (currentChar == '\n') {

				newLines++;
				lastCharacterWasNewLine = true;

				if (lastCharacterWasCarriageReturn) {
					lastCharacterWasCarriageReturn = false;
					newLinesWithCarriageReturns++;
				}
			} else {
				lastCharacterWasNewLine = false;
			}

			if (currentChar == '\r') {
				carriageReturns++;
				lastCharacterWasCarriageReturn = true;
				continue;
			} else {
				lastCharacterWasCarriageReturn = false;
			}

			if (Character.isLetter(currentChar)) {

				insideWord = true;

			} else {

				if (insideWord) {
					words++;
					insideWord = false;
				}
			}
		}

		if (insideWord) {
			words++;
		}

		if (!lastCharacterWasNewLine) {
			newLines++;
		}

		StringSupport.TextCharacteristics characteristics = (new StringSupport()).new TextCharacteristics(
				text.length(), words, newLines,
				newLinesWithCarriageReturns > 0 ? TextCharacteristics.DOS
						: TextCharacteristics.UNIX);

		return characteristics;
	}

	/**
	 * Bean to indicate a Caret position.
	 * 
	 * @author Carlos Fernando Bella Cruz - <a
	 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
	 */
	public class CaretPosition {

		private int line = 0;
		private int column = 0;

		/**
		 * Default constructor
		 * 
		 * @param line
		 *            A line
		 * @param column
		 *            A column
		 */
		public CaretPosition(int line, int column) {

			super();

			this.line = line;
			this.column = column;
		}

		/**
		 * @return the line
		 */
		public int getLine() {
			return line;
		}

		/**
		 * @return the column
		 */
		public int getColumn() {
			return column;
		}
	}

	/**
	 * Indicate the caret position.
	 * 
	 * @param text
	 *            The text.
	 * @param position
	 *            The position.
	 * @return The line and column
	 */
	public StringSupport.CaretPosition getCaretPosition(String text,
			int position) {

		int line = 1;
		int column = 0;
		for (int i = 0; i < position; i++) {

			if (text.charAt(i) == '\n') {
				line++;
				column = 1;
			} else {
				column++;
			}
		}

		return new CaretPosition(line, column);
	}

	/**
	 * Inform the number of characters per line.
	 * 
	 * @param text
	 *            The text.
	 * @return The number of characters per lines.
	 */
	public static int[] getCharactersPerLine(String text) {

		ArrayList lines = new ArrayList();

		int characters = 0;
		for (int i = 0; i < text.length(); i++) {

			char currentChar = text.charAt(i);

			if (currentChar == '\n') {

				lines.add(new Integer(characters));
				characters = 0;
			} else {

				characters++;
			}
		}

		int resultLines[] = new int[lines.size()];
		for (int i = 0; i < lines.size(); i++) {

			Integer numberOfCharacters = (Integer) lines.get(i);
			resultLines[i] = numberOfCharacters.intValue();
		}
		return resultLines;
	}

	/**
	 * Analyze the line provided and break into separated commands. This method
	 * is aware of elements using quotation marks.
	 * 
	 * @param line
	 *            The line to be analyzed.
	 * @return The command line elements.
	 */
	public static String[] breakIntoCommandLineElements(String line) {

		ArrayList elements = new ArrayList();

		String currentElement = new String();
		boolean insideQuotes = false;
		for (int i = 0; i < line.length(); i++) {

			char currentChar = line.charAt(i);

			if ('"' == currentChar) {

				if (insideQuotes) {

					elements.add(currentElement);

					currentElement = new String();
					insideQuotes = false;

					continue;

				} else {

					insideQuotes = true;
					continue;
				}
			}

			if (insideQuotes) {
				// Inside quotes than include everything.
				currentElement += currentChar;
				continue;
			}

			if (Character.isWhitespace(currentChar)) {

				// If current element has content.
				if ("".equals(currentElement)) {

					// There is no current element.
					// Ignore spaces.
					continue;

				} else {

					// Add to the list.
					elements.add(currentElement);
					currentElement = new String();
				}

			} else {

				currentElement += currentChar;
			}
		}

		// Add the last command if it exist.
		if (!"".equals(currentElement.trim())) {

			elements.add(currentElement.trim());
		}

		String elementsArray[] = new String[elements.size()];
		elementsArray = (String[]) elements.toArray(elementsArray);

		return elementsArray;
	}

	/**
	 * Provide the DOS type new line.
	 * @return The DOS type new line.
	 */
	public static String getDOSNewLine() {
		
		return "\r\n";
	}
	
	/**
	 * Provide the Unix type new line.
	 * @return The Unix type new line.
	 */
	public static String getUnixNewLine() {
		
		return "\n";
	}
	
	/**
	 * Switch the line type between DOS and UNIX
	 * 
	 * @param text
	 *            The text to be converted.
	 * @return The text converted.
	 */
	public static String convertLineType(String text) {

		StringBuffer out = new StringBuffer();

		boolean isDOS2UNIX = false;
		boolean isUNIX2DOS = false;

		boolean isLastCharacterCarriageReturn = false;

		for (int i = 0; i < text.length(); i++) {

			char currentChar = text.charAt(i);

			if (isDOS2UNIX || isUNIX2DOS) {
				// Change mode.

				if (isDOS2UNIX) {
					if (currentChar == '\r') {
						continue;
					}
				}

				if (isUNIX2DOS) {
					if (currentChar == '\n') {
						out.append(getDOSNewLine());
						continue;
					}
				}

			} else {
				// Identification mode.

				if (currentChar == '\r') {
					isLastCharacterCarriageReturn = true;
					continue;
				} else {

					// if it is not an isolated carriage return.
					if (currentChar != '\n') {
						isLastCharacterCarriageReturn = false;
					}
				}

				if (currentChar == '\n') {
					if (isLastCharacterCarriageReturn) {
						isDOS2UNIX = true;
					} else {
						isUNIX2DOS = true;
						out.append(getDOSNewLine());
						continue;
					}
				}
			}

			out.append(currentChar);
		}
		return out.toString();
	}

	/**
	 * Creates a new copy array starting from a specific position.
	 * 
	 * @param original
	 *            The original array.
	 * @param statingPosition
	 *            The new position to start copying.
	 * @return The new array containing the elements from the startingPosition
	 *         until the end of the original array.
	 */
	public static String[] arrayAtPosition(String[] original,
			int statingPosition) {

		int newSize = original.length - statingPosition;

		// Safeguard array size.
		if (newSize <= 0) {
			return new String[0];
		}

		String newArray[] = new String[newSize];

		for (int i = 0; i < newArray.length; i++) {
			newArray[i] = original[statingPosition + i];
		}
		return newArray;
	}

	/**
	 * Indicate the character position of the beginning of provided line.
	 * 
	 * @param text
	 *            The text to be analyzed.
	 * @param line
	 *            The line required.
	 * @return The position.
	 */
	public static int getPositionOfBeginnintOfLine(StringBuffer text, int line) {

		int currentLine = 1;
		for (int i = 0; i < text.length(); i++) {

			char currentChar = text.charAt(i);

			if (currentLine == line) {

				return i;
			}

			if (currentChar == '\n') {

				currentLine++;

				continue;

			}
		}

		return -1;
	}

	/**
	 * Indicate the character position of the end of provided line.
	 * 
	 * @param text
	 *            The text to be analyzed.
	 * @param line
	 *            The line required.
	 * @return The position.
	 */
	public static int getPositionOfTheEndOfLine(StringBuffer text, int line) {

		int currentLine = 1;
		for (int i = 0; i < text.length(); i++) {

			char currentChar = text.charAt(i);

			if (currentChar == '\n') {

				if (currentLine == line) {

					return i + 1;
				}

				currentLine++;

				continue;

			}
		}

		return -1;
	}

	/**
	 * Convert a text into a string that can be used for file names
	 * 
	 * @param text
	 *            A text to be converted.
	 * @return A valid string to be used as a file name.
	 */
	public static String getConvertNonFileCharacters(String text) {

		String file = new String();

		// Eliminate http header.
		if (text.startsWith("http://")) {
			text = text.substring(7, text.length());
		}

		for (int i = 0; i < text.length(); i++) {

			char character = text.charAt(i);

			if (character == '\\' || character == '/' || character == '%'
					|| character == '@' || character == '=' || character == '&'
					|| character == '.') {
				file += '_';
				continue;
			}

			file += character;
		}

		file += ".txt";

		return file;
	}
}
