/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.util;

import java.security.InvalidParameterException;
import java.util.Date;

import javax.swing.swingworker.SwingWorker;

/**
 * It represents a Task to be executed and monitored. It can be a 
 * finite or an infinite task. If it is a finite task than the
 * the progress can be monitored by a progress bar and the result
 * can be captured at the end. If it is an infinite task than
 * intermediary results can be retrieved at any time and the
 * progress can be monitored by units, that is a simple number.
 *  
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public abstract class Task {

    // Main reference to the internal task control with SwingWorker support.
    private InternalTask internalTask;

    // A simple and short task name.
    private String taskName = new String();

    // Any parameter that can be passed to the task
    private Object parameter = null;

    // If the task is an infinite task than it can have intermediary task results.
    private Object intermediaryResult = null;
    // A flag to prevent the caller to retrieve the same intermediary task twice.
    private boolean intermediaryResultAvailable = false;

    // If the task is finite than the caller can set the minimum and maximum progress.
    private int lowerLimit = 0;
    private int upperLimit = 100;
    // Indication to the corrent progress. It should be within the minimum and maximum.
    private int currentProgress = 0;
    // A task can be done in more than one step
    private int steps = 1;
    private int currentStep = 1;

    // A task can be in three states. Or it is running, or finished or interrupted.
    private boolean finished = false;
    private boolean interrupted = false;

    // The time controls of a task. It is used to calculate the elapsed time.
    private Date startDate = null;
    private Date endDate = null;
    private Date lastStepDate = null;

    // Indicate if the result of a finished finite task has been retrieved
    private boolean resultRestrieved = false;

    // Indicate if a task has been interrupted because of an exception.
    // The caller can retrieve this exception only once.
    private Throwable taskException = null;
    // Indicate that this task has been interrupted by an exception.
    private boolean interruptedByException = false;

    /**
     * Constructor with task name.
     * @param taskName The task name.
     */
    public Task(String taskName) {

        super();
        this.taskName = taskName;
    }

    /**
     * Constructor with the task name and a parameter.
     * @param taskName The task name.
     * @param parameter The parameter to the task.
     */
    public Task(String taskName, Object parameter) {

        super();
        this.taskName = taskName;
        this.parameter = parameter;
    }

    /**
     * Inform the task name.
     * @return The task name.
     */
    String getTaskName() {

        return taskName;
    }

    /**
     * Set the lower limit for the progress control.
     * @param minimum the minimum to set
     */
    protected void setLowerLimit(int lowerLimit) {

        this.lowerLimit = lowerLimit;
    }

    /**
     * Inform the progress lower limit.
     * @return the minimum The lower limit.
     */
    int getLowerLimit() {

        return lowerLimit;
    }

    /**
     * Set the upper limit for the progress control.
     * @param maximum the maximum to set
     */
    protected void setUpperLimit(int upperLimit) {

        this.upperLimit = upperLimit;
    }

    /**
     * Inform the progress upper limit.
     * @return the maximum The upper limit.
     */
    int getUpperLimit() {

        return upperLimit;
    }

    /**
     * Inform the current step.
     * @return the currentStep
     */
    int getCurrentStep() {

        return currentStep;
    }

    /**
     * Set the current step.
     * @param currentStep the currentStep to set
     */
    protected void setCurrentStep(int currentStep) {

        if (this.currentStep != currentStep) {
            lastStepDate = new Date();
        }

        this.currentStep = currentStep;
    }

    /**
     * Inform the total number of steps.
     * @return the steps
     */
    int getSteps() {

        return steps;
    }

    /**
     * Set the total number of steps.
     * @param steps the steps to set
     */
    protected void setSteps(int steps) {

        this.steps = steps;
    }

    /**
     * Set the current task progress;
     * @param currentProgress the currentProgress to set
     */
    protected void setCurrentProgress(int currentProgress) {

        // The current progress should be within the limits.
        if (currentProgress < lowerLimit || currentProgress > upperLimit)
            throw new InvalidParameterException(InternalResource
                    .get(InternalResource.PROGRESS_LIMIT));

        this.currentProgress = currentProgress;
    }

    /**
     * Inform the current task progress.
     * @return the currentProgress
     */
    int getCurrentProgress() {

        return currentProgress;
    }

    /**
     * Inform the current progress in a percentage from 0 to 100.
     * @return the currentProgress
     */
    int getCurrentProgressNormalized() {

        int current = currentProgress - lowerLimit;
        int work = upperLimit - lowerLimit;

        int progressBar = lowerLimit;
        if (work != 0) {
            progressBar = current * 100 / work;
        }

        return progressBar;
    }

    /**
     * Provide the result of the task execution. This is used for finite tasks.
     * @return The task result.
     */
    Object getResult() {

        return internalTask.get();
    }

    /**
     * Store the intermediary result of a infinite task.
     * @param intermediaryResult the intermediaryResult to set
     */
    protected void setIntermediaryResult(Object intermediaryResult) {

        this.intermediaryResult = intermediaryResult;

        intermediaryResultAvailable = true;
    }

    /**
     * Provide the current intermediary result. This is used for infinite tasks.
     * @return The intermediary result.
     */
    Object getIntermediaryResult() {

        intermediaryResultAvailable = false;

        return intermediaryResult;

    }

    /**
     * Force the stop of the task by interrupting it.
     */
    void stopTask() {

        internalTask.interrupt();
        interrupted = true;
        endDate = new Date();
    }

    /**
     * Inform that the final result from the task has been retrieved.
     * @param resultRestrieved the resultRestrieved to set
     */
    void setResultRestrieved(boolean resultRestrieved) {

        this.resultRestrieved = resultRestrieved;
    }

    /**
     * Inform if the final result from the task has been retrieved.
     * @return the resultRestrieved The indication if the result has been retreived or not.
     */
    boolean isResultRestrieved() {

        return resultRestrieved;
    }

    /**
     * Indicate if there is any intermediary result. This method
     * applies for infinite type task.
     * @return the intermediaryResultAvailable
     */
    boolean isIntermediaryResultAvailable() {

        return intermediaryResultAvailable;
    }

    /**
     * Inform the current elapsed time. This is the current time minus the start date.
     * @return the number of seconds since the task has been started.
     */
    long getElapsedTime() {

        // If the task is finished than there are both the start and end time.
        if (startDate != null && endDate != null) {
            return (endDate.getTime() - startDate.getTime()) / 1000;
        }

        // If the task is running than there is only the start time.
        if (startDate != null) {
            return ((new Date()).getTime() - startDate.getTime()) / 1000;
        }

        return 0l;
    }

    /**
     * Inform the estimated diratin to the end of the task.
     * @return the estimated number of seconds necessary to end the task.
     */
    long getRemainingTime() {

        // If the task is finished.
        if (lastStepDate != null && endDate != null) {
            return 0l;
        }

        // If the task is running.
        if (lastStepDate != null) {

            long currentDate = new Date().getTime();
            long elapsedTime = currentDate - lastStepDate.getTime();
            long totalWork = upperLimit - lowerLimit;
            long currentExecution = currentProgress - lowerLimit;

            // Avoid division by zero.
            if (currentExecution != 0l) {

                long expectedDuration = elapsedTime * totalWork
                    / currentExecution;
                long remaining = (expectedDuration - (currentDate - lastStepDate
                        .getTime())) / 1000;

                return remaining;
            }
        }

        return 0l;
    }

    /**
     * Indicate if the task has finished.
     * @return the finished The indication if the task has finished.
     */
    boolean isFinished() {

        return finished;
    }

    /**
     * Indicate if the task has been interrupted.
     * @return the interrupted The indication if the task has been interrupted.
     */
    boolean isInterrupted() {

        return interrupted;
    }

    /**
     * Indicate that this task has been interrupted by an exception.
     * @return The indication if this task has been interrupted by an exception.
     */
    boolean isInterruptedByException() {

        return interruptedByException;
    }

    /**
     * Inform if the task has been interrupted by an exception.
     * @return the taskException The exception if it has ocurred or null if not.
     */
    Throwable getTaskException() {

        Throwable returnException = this.taskException;
        this.taskException = null;
        return returnException;
    }

    /**
     * This is the internal control of the running task.
     *  
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    private class InternalTask extends SwingWorker {

        /* (non-Javadoc)
         * @see javax.swing.swingworker.SwingWorker#construct()
         */
        public Object construct() {

            try {

                Object returnObject = run(parameter);

                finished = true;

                endDate = new Date();

                return returnObject;

            }
            catch(InterruptedException e) {

                interrupted = true;
                endDate = new Date();

                return InternalResource.get(InternalResource.TASK_INTERRUPTED);
            }
            catch(Throwable t) {

                internalTask.interrupt();
                interrupted = true;
                endDate = new Date();

                taskException = t;
                interruptedByException = true;

                return InternalResource.get(InternalResource.TASK_INTERRUPTED);
            }
        }
    }

    /**
     * Verify if the thread is interrupted, so the user can stop the task.
     * The task should call this from time to time.
     * @throws InterruptedException
     */
    protected void verifyInterrupted() throws InterruptedException {

        if (Thread.interrupted()) {
            throw new InterruptedException();
        }
    }

    /**
     * This is the method any task has to implement and that is
     * used to execute any required task.
     */
    protected abstract Object run(Object parameter) throws InterruptedException;

    /**
     * Indicate to start the execution of this task.
     */
    void start() {

        internalTask = new InternalTask();
        internalTask.start();
        lastStepDate = startDate = new Date();
    }
}
